/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document.bescheid;

import java.time.LocalDate;
import java.util.List;
import java.util.Optional;

import de.ozgcloud.document.bescheid.vorgang.VorgangId;
import de.ozgcloud.common.binaryfile.FileId;
import lombok.Builder;
import lombok.Getter;
import lombok.Singular;

@Builder(toBuilder = true)
@Getter
public class Bescheid {

	public static final String FIELD_VORGANG_ID = "vorgangId";
	public static final String FIELD_CREATED_BY = "createdBy";

	public static final String FIELD_BEWILLIGT = "bewilligt";
	public static final String FIELD_STATUS = "status";
	public static final String FIELD_BESCHEID_DOCUMENT = "bescheidDocument";
	public static final String FIELD_BESCHIEDEN_AM = "beschiedenAm";
	public static final String FIELD_SEND_BY = "sendBy";

	public static final String FIELD_NACHRICHT_TEXT = "nachrichtText";
	public static final String FIELD_NACHRICHT_SUBJECT = "nachrichtSubject";
	public static final String FIELD_ATTACHMENTS = "attachments";
	public static final String FIELD_SENT_INFO = "sentInfo";

	private BescheidId id;
	private long version;

	private VorgangId vorgangId;
	private UserId createdBy;

	private boolean bewilligt;
	private Status status;
	private FileId bescheidDocument;
	private LocalDate beschiedenAm;
	private SendBy sendBy;

	private SentInfo sentInfo;

	@Builder.Default
	private Optional<String> nachrichtSubject = Optional.empty();
	@Builder.Default
	private Optional<String> nachrichtText = Optional.empty();

	@Singular
	private List<FileId> attachments;

	public enum Status {
		DRAFT, SENT;

		public boolean not(Object value) {
			return !hasValue(value);
		}

		public boolean hasValue(Object value) {
			return this.name().equalsIgnoreCase(String.valueOf(value));
		}
	}

	public enum SendBy {
		NACHRICHT, MANUAL;

		public boolean notValue(Object sendByValue) {
			return !hasValue(sendByValue);
		}

		public boolean hasValue(Object sendByValue) {
			return this.name().equalsIgnoreCase(String.valueOf(sendByValue));
		}

	}
}