/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document.bescheid.vorgang;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.ReportingPolicy;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.vorgang.GrpcEingang;
import de.ozgcloud.vorgang.vorgang.GrpcFormData;
import de.ozgcloud.vorgang.vorgang.GrpcPostfachAddress;
import de.ozgcloud.vorgang.vorgang.GrpcServiceKonto;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangWithEingang;

@Mapper(unmappedTargetPolicy = ReportingPolicy.ERROR, //
		uses = { FormDataEntryMapper.class, GrpcObjectMapper.class })
interface BescheidVorgangMapper {

	@Mapping(target = "serviceKonto", source = "header.serviceKonto")
	@Mapping(target = "vorgangNummer", source = "nummer")
	@Mapping(target = "vorgangName", source = "name")
	Vorgang mapVorgang(GrpcVorgangWithEingang vorgang);

	@Mapping(target = "postfachAddress", ignore = true)
	@Mapping(target = "postfachAddresses", source = "postfachAddressesList")
	Vorgang.ServiceKonto mapServiceKonto(GrpcServiceKonto serviceKonto);

	Vorgang.PostfachAddress mapAddress(GrpcPostfachAddress address);

	VorgangId mapVorgangId(String vorgangId);

	Vorgang.Eingang mapEingang(GrpcEingang eingang);

	default Collection<FormDataEntry> mapFormData(GrpcFormData value) {
		var result = new ArrayList<FormDataEntry>();
		var entryMapper = Mappers.getMapper(FormDataEntryMapper.class);

		result.addAll(value.getFieldList().stream().map(entryMapper::fromGrpc).toList());
		result.addAll(value.getFormList().stream().map(entryMapper::fromGrpc).toList());

		return Collections.unmodifiableList(result);
	}

}
