/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document.bescheid.vorgang;

import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import lombok.Builder;
import lombok.Getter;
import lombok.Singular;
import lombok.ToString;

@Builder
@Getter
@ToString
public class Vorgang {

	@JsonIgnore
	private VorgangId id;
	private long version;

	private String vorgangName;

	private String vorgangNummer;
	private String aktenzeichen;

	@JsonIgnore
	@ToString.Exclude
	private ServiceKonto serviceKonto;
	@ToString.Exclude
	private Eingang eingang;

	@Builder
	@Getter
	public static class ServiceKonto {

		public static final String FIELD_SERVICEKONTO_TYPE = "serviceKontoType";
		public static final String FIELD_POSTFACH_ADDRESS = "postfachAddress";

		private String type;
		@Singular
		private List<PostfachAddress> postfachAddresses;
	}

	@Builder
	@Getter
	public static class PostfachAddress {

		public static final String FIELD_VERSION = "version";
		public static final String FIELD_TYPE = "type";
		public static final String FIELD_IDENTIFIER = "identifier";

		private String version;
		private int type;
		private Map<String, Object> identifier;
	}

	@Builder
	@Getter
	static class Eingang {
		private Antragsteller antragsteller;
		private ZustaendigeStelle zustaendigeStelle;

		@Singular("formData")
		@JsonSerialize(using = FormDataEntryListSerializer.class)
		private List<FormDataEntry> formData;
	}

	@Builder
	@Getter
	@JsonInclude(Include.NON_EMPTY)
	static class Antragsteller {
		private String anrede;
		private String nachname;
		private String vorname;
		private String geburtsdatum;
		private String geburtsort;
		private String geburtsname;
		private String email;
		private String telefon;
		private String strasse;
		private String hausnummer;
		private String plz;
		private String ort;
	}

	@Builder
	@Getter
	static class ZustaendigeStelle {
		private String organisationseinheitenId;
		private String email;
	}

}
