/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document.bescheid;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

class SentInfoMapperTest {

	private final SentInfoMapper mapper = Mappers.getMapper(SentInfoMapper.class);

	@DisplayName("Map to")
	@Nested
	class TestMapTo {

		@Test
		void shouldMap() {
			var mappedSentInfo = mapper.mapTo(SentInfoTestFactory.create());

			assertThat(mappedSentInfo).usingRecursiveComparison().isEqualTo(GrpcSentInfoTestFactory.create());
		}
	}

	@DisplayName("Map to SentInfo")
	@Nested
	class TestToSentInfo {

		@Test
		void shouldMap() {
			var mappedSentInfo = mapper.toSentInfo(SentInfoTestFactory.asMap());

			assertThat(mappedSentInfo).usingRecursiveComparison().isEqualTo(SentInfoTestFactory.create());
		}
	}

	@DisplayName("As map")
	@Nested
	class TestAsMap {

		@Test
		void shouldMap() {
			var mappedSentInfo = mapper.toMap(SentInfoTestFactory.create());

			assertThat(mappedSentInfo).isEqualTo(SentInfoTestFactory.asMap());
		}
	}
}
