/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document.bescheid.smartdocuments;

import static org.assertj.core.api.Assertions.*;

import java.io.File;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.ActiveProfiles;

import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.user.UserProfileMapper;
import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.document.bescheid.BescheidRequestTestFactory;
import de.ozgcloud.document.bescheid.BescheidTestApplication;
import de.ozgcloud.document.bescheid.vorgang.VorgangTestFactory;

@SpringBootTest(classes = BescheidTestApplication.class)
@ITCase
@ActiveProfiles({ "itcase", "local" })
class SmartDocumentsBescheidRemoteServiceITCase {

	@Autowired
	private SmartDocumentsBescheidRemoteService remoteService;
	@MockBean
	private OzgCloudCommandService ozgCloudCommandService;
	@MockBean
	private CommandMapper commandMapper;
	@MockBean
	private UserProfileMapper userProfileMapper;

	@Disabled("This test request SmartDocuments service")
	@Test
	void createBescheid() {
		var bescheid = remoteService.create(BescheidRequestTestFactory.create(), VorgangTestFactory.create());

		System.out.println(bescheid.getBescheidFileName());
		System.out.println(bescheid.getBescheidFile().getAbsolutePath());
		assertThat(bescheid.getBescheidFileName()).isNotEmpty();
	}

	@Nested
	class TestGetNachrichtenText {

		private static final String NACHRICHTEN_TEXT = "Nachrichtentext";
		private static final String FIELD_TEMPLATE_TEXT = "Nachrichten Field Text";

		@Test
		void shouldGetNachrichtenText() {
			var nachrichtenText = remoteService.extractTextFromXmlFile(loadFile("SD_answer.xml"));

			assertThat(nachrichtenText).contains(NACHRICHTEN_TEXT);
		}

		@Test
		void shouldGetFieldTemplateText() {
			var fieldTemplateText = remoteService.extractTextFromXmlFile(loadFile("SD_answer_field_template.xml"));

			assertThat(fieldTemplateText).contains(FIELD_TEMPLATE_TEXT);
		}

		@Test
		void shouldExpectMissingTextNode() {
			File xmlFileWithoutText = TempFileUtils.writeTmpFile(TestUtils.loadFile("SD_answer_without_text.xml"));

			var text = remoteService.extractTextFromXmlFile(xmlFileWithoutText);

			assertThat(text).isEmpty();
		}

		private File loadFile(String fileName) {
			return TempFileUtils.writeTmpFile(TestUtils.loadFile(fileName));
		}
	}
}
