/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document.common.attached_item;

import static de.ozgcloud.document.bescheid.SentInfoTestFactory.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.ozgcloud.document.bescheid.Bescheid;
import de.ozgcloud.document.bescheid.BescheidTestFactory;
import de.ozgcloud.document.bescheid.SentInfo;
import de.ozgcloud.document.bescheid.SentInfoTestFactory;
import de.ozgcloud.document.bescheid.common.user.UserProfileTestFactory;
import de.ozgcloud.document.bescheid.vorgang.VorgangTestFactory;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.document.common.attached_item.AttachedItem.AttachedItemBuilder;
import de.ozgcloud.common.binaryfile.FileId;
import de.ozgcloud.document.Document;
import de.ozgcloud.document.DocumentService;
import de.ozgcloud.document.DocumentTestFactory;

public class AttachedItemTestFactory {

	public static final String ID = "bescheid-item-id";
	public static final long VERSION = 10L;
	public static final String CLIENT = "client";
	public static final Map<String, Object> SENT_INFO = SentInfoTestFactory.asMap();

	public static AttachedItem create() {
		return createBuilder().build();
	}

	public static AttachedItem createBescheid() {
		return createBescheidBuilder().build();
	}

	public static AttachedItem.AttachedItemBuilder createBescheidBuilder() {
		return createBuilder()
				.itemName(AttachedItemService.BESCHEID_ITEM_NAME)
				.item(createBescheidItem());
	}

	public static Map<String, Object> createBescheidItem() {
		var item = new HashMap<String, Object>();
		item.put(Bescheid.FIELD_STATUS, BescheidTestFactory.STATUS.name());
		item.put(Bescheid.FIELD_BESCHIEDEN_AM, BescheidTestFactory.BESCHIEDEN_AM_STR);
		item.put(Bescheid.FIELD_BEWILLIGT, BescheidTestFactory.BEWILLIGT);
		item.put(Bescheid.FIELD_BESCHEID_DOCUMENT, DocumentTestFactory.ID);
		item.put(Bescheid.FIELD_ATTACHMENTS, new ArrayList<>(List.of(BescheidTestFactory.ATTACHMENT_STR, BescheidTestFactory.ATTACHMENT_STR)));
		item.put(Bescheid.FIELD_SEND_BY, BescheidTestFactory.SEND_BY.name());
		item.put(Bescheid.FIELD_NACHRICHT_TEXT, BescheidTestFactory.NACHRICHT_TEXT);
		item.put(Bescheid.FIELD_NACHRICHT_SUBJECT, BescheidTestFactory.NACHRICHT_SUBJECT);
		item.put(Bescheid.FIELD_CREATED_BY, UserProfileTestFactory.ID_STR);
		item.put(Bescheid.FIELD_SENT_INFO, SENT_INFO);
		return item;
	}

	public static AttachedItem createBescheidWithoutDocument() {
		return createBescheidWithoutDocumentBuilder().build();
	}

	public static AttachedItem.AttachedItemBuilder createBescheidWithoutDocumentBuilder() {
		return createBuilder()
				.itemName(AttachedItemService.BESCHEID_ITEM_NAME)
				.item(createBescheidItemWithoutDocument());
	}

	private static Map<String, Object> createBescheidItemWithoutDocument() {
		var item = new HashMap<String, Object>();
		item.put(AttachedItem.PROPERTY_ID, AttachedItemTestFactory.ID);
		item.put(AttachedItem.PROPERTY_VERSION, AttachedItemTestFactory.VERSION);
		item.put(Bescheid.FIELD_VORGANG_ID, VorgangTestFactory.ID);
		item.put(Bescheid.FIELD_CREATED_BY, UserProfileTestFactory.ID);
		item.put(Bescheid.FIELD_STATUS, BescheidTestFactory.STATUS.name());
		item.put(Bescheid.FIELD_BESCHIEDEN_AM, BescheidTestFactory.BESCHIEDEN_AM);
		item.put(Bescheid.FIELD_BEWILLIGT, BescheidTestFactory.BEWILLIGT);
		item.put(Bescheid.FIELD_BESCHEID_DOCUMENT, FileId.from(DocumentTestFactory.ID));
		item.put(Bescheid.FIELD_ATTACHMENTS, BescheidTestFactory.ATTACHMENTS);
		item.put(Bescheid.FIELD_SEND_BY, BescheidTestFactory.SEND_BY.name());
		item.put(Bescheid.FIELD_NACHRICHT_TEXT, BescheidTestFactory.NACHRICHT_TEXT);
		item.put(Bescheid.FIELD_NACHRICHT_SUBJECT, BescheidTestFactory.NACHRICHT_SUBJECT);
		item.put(Bescheid.FIELD_SENT_INFO, Map.of(
				SentInfo.FIELD_SENT_BY, SENT_BY,
				SentInfo.FIELD_SENT_AT, SENT_AT_STR));
		return item;
	}

	public static AttachedItem createDocument() {
		return createDocumentBuilder().build();
	}

	public static AttachedItem.AttachedItemBuilder createDocumentBuilder() {
		return createBuilder()
				.itemName(DocumentService.DOCUMENT_ITEM_NAME)
				.item(createDocumentItem());
	}

	public static HashMap<String, Object> createDocumentItem() {
		var map = new HashMap<String, Object>();
		map.put(Document.FIELD_DOCUMENT_TYPE, DocumentService.DOCUMENT_TYPE);
		map.put(Document.FIELD_DOCUMENT_FILE, DocumentTestFactory.DOCUMENT_FILE);
		map.put(Document.FIELD_NACHRICHT_TEXT, DocumentTestFactory.NACHRICHT_TEXT);
		map.put(Document.FIELD_NACHRICHT_SUBJECT, DocumentTestFactory.NACHRICHT_SUBJECT);
		return map;
	}

	public static AttachedItemBuilder createBuilder() {
		return AttachedItem.builder()
				.id(ID)
				.version(VERSION)
				.client(CLIENT)
				.vorgangId(CommandTestFactory.VORGANG_ID);
	}
}